Error

Struct Error 

pub struct Error(/* private fields */);
Error type used by Topcoat APIs.

This is a thin wrapper around anyhow::Error that provides a shared application error type while still allowing callers to inspect, downcast, and mutate the underlying error when needed.

Implementations

Source

impl Error

pub fn downcast<E>(self) -> Result<E, Self>
where E: Display + Debug + Send + Sync + 'static,

Attempt to downcast the error object to a concrete type.

Errors

Returns Err(Self) if the stored error is not an instance of E, handing back the original error unchanged.

pub fn downcast_ref<E>(&self) -> Option<&E>
where E: Display + Debug + Send + Sync + 'static,

Downcast this error object by reference.

pub fn downcast_mut<E>(&mut self) -> Option<&mut E>
where E: Display + Debug + Send + Sync + 'static,

Downcast this error object by mutable reference.

Methods from Deref<Target = Error>

pub fn backtrace(&self) -> &Backtrace

Get the backtrace for this Error.

In order for the backtrace to be meaningful, one of the two environment variables RUST_LIB_BACKTRACE=1 or RUST_BACKTRACE=1 must be defined and RUST_LIB_BACKTRACE must not be 0. Backtraces are somewhat expensive to capture in Rust, so we don’t necessarily want to be capturing them all over the place all the time.

  • If you want panics and errors to both have backtraces, set RUST_BACKTRACE=1;
  • If you want only errors to have backtraces, set RUST_LIB_BACKTRACE=1;
  • If you want only panics to have backtraces, set RUST_BACKTRACE=1 and RUST_LIB_BACKTRACE=0.
Stability

Standard library backtraces are only available when using Rust ≥ 1.65. On older compilers, this function is only available if the crate’s “backtrace” feature is enabled, and will use the backtrace crate as the underlying backtrace implementation. The return type of this function on old compilers is &(impl Debug + Display).

[dependencies]
anyhow = { version = "1.0", features = ["backtrace"] }
pub fn chain(&self) -> Chain<'_>

An iterator of the chain of source errors contained by this Error.

This iterator will visit every error in the cause chain of this error object, beginning with the error that this error object was created from.

Example
use anyhow::Error;
use std::io;

pub fn underlying_io_error_kind(error: &Error) -> Option<io::ErrorKind> {
    for cause in error.chain() {
        if let Some(io_error) = cause.downcast_ref::<io::Error>() {
            return Some(io_error.kind());
        }
    }
    None
}
pub fn root_cause(&self) -> &(dyn Error + 'static)

The lowest level cause of this error — this error’s cause’s cause’s cause etc.

The root cause is the last error in the iterator produced by chain().

pub fn is<E>(&self) -> bool
where E: Display + Debug + Send + Sync + 'static,

Returns true if E is the type held by this error object.

For errors with context, this method returns true if E matches the type of the context C or the type of the error on which the context has been attached. For details about the interaction between context and downcasting, see here.

pub fn downcast_ref<E>(&self) -> Option<&E>
where E: Display + Debug + Send + Sync + 'static,

Downcast this error object by reference.

Example
// If the error was caused by redaction, then return a tombstone instead
// of the content.
match root_cause.downcast_ref::<DataStoreError>() {
    Some(DataStoreError::Censored(_)) => Ok(Poll::Ready(REDACTED_CONTENT)),
    None => Err(error),
}
pub fn downcast_mut<E>(&mut self) -> Option<&mut E>
where E: Display + Debug + Send + Sync + 'static,

Downcast this error object by mutable reference.